function [xx, yy, zz, xy, xz, yz, normVec] = tri_strain(x, y, z, sx, sy, sz, ss, ds, ts, pr)
%
% tri_strain.m
%
% Calculates strains due to slip on a triangular dislocation in an
% elastic half space utilizing the symbolically differentiated
% displacement gradient tensor derived from the expressions for
% the displacements due to an angular dislocation in an elastic half
% space (Comninou and Dunders, 1975).
%
% Arguments
%  sx : x-coordinates of observation points
%  sy : y-coordinates of observation points
%  sz : z-coordinates of observation points
%  x  : x-coordinates of triangle vertices.
%  y  : y-coordinates of triangle vertices.
%  z  : z-coordinates of triangle vertices.
%  pr : Poisson's ratio
%  ss : strike slip displacement
%  ts : tensile slip displacement
%  ds : dip slip displacement
%
% Returns
%  ij  : strain components (xx, yy, zz, xy, xz, yz)
%


% Calculate the slip vector in XYZ coordinates
normVec                      = cross([x(2);y(2);z(2)]-[x(1);y(1);z(1)], [x(3);y(3);z(3)]-[x(1);y(1);z(1)]);
normVec                      = normVec./norm(normVec);
if (normVec(3) < 0) % Enforce clockwise circulation
   normVec                   = -normVec;
   [x(2) x(3)]               = swap(x(2), x(3));
   [y(2) y(3)]               = swap(y(2), y(3));
   [z(2) z(3)]               = swap(z(2), z(3));
end
strikeVec                    = [-sin(atan2(normVec(2),normVec(1))) cos(atan2(normVec(2),normVec(1))) 0];
dipVec                       = cross(normVec, strikeVec);
slipComp                     = [ss ds ts];
slipVec                      = [strikeVec(:) dipVec(:) normVec(:)] * slipComp(:);

% Solution vectors
xx                           = zeros(size(sx));
yy                           = xx;
zz                           = xx;
xy                           = xx;
xz                           = xx;
yz                           = xx;

% Add a copy of the first vertex to the vertex list for indexing
x(4)                         = x(1);
y(4)                         = y(1);
z(4)                         = z(1);

for iTri = 1:3
   % Calculate strike and dip of current leg
   strike                   = 180/pi*(atan2(y(iTri+1)-y(iTri), x(iTri+1)-x(iTri)));
   segMapLength             = sqrt((x(iTri)-x(iTri+1))^2 + (y(iTri)-y(iTri+1))^2);
   [rx ry]                  = RotateXyVec(x(iTri+1)-x(iTri), y(iTri+1)-y(iTri), -strike);
   dip                      = 180/pi*(atan2(z(iTri+1)-z(iTri), rx));
   
   if dip >= 0
      beta                  = pi/180*(90-dip);
      if beta > pi/2
         beta               = pi/2-beta;
      end
   else
      beta                  = -pi/180*(90+dip);
      if beta < -pi/2
         beta = pi/2-abs(beta);
      end
   end
   ssVec                    = [cos(strike/180*pi) sin(strike/180*pi) 0];
   dsVec                    = [-sin(strike/180*pi) cos(strike/180*pi) 0];
   tsVec                    = cross(ssVec, dsVec);
   lss                      = dot(slipVec, ssVec);
   lts                      = dot(slipVec, tsVec);
   lds                      = dot(slipVec, dsVec);

   if (abs(beta) > 0.000001) && (abs(beta-pi) > 0.000001)
      % First angular dislocation
      [sx1 sy1]                 = RotateXyVec(sx-x(iTri), sy-y(iTri), -strike);
      [a11 a22 a33 a12 a13 a23] = advs(sx1, sy1, sz-z(iTri), z(iTri), beta, pr, lss, lds, lts);
                                   
      % Second angular dislocation
      [sx2 sy2]                 = RotateXyVec(sx-x(iTri+1), sy-y(iTri+1), -strike); 
      [b11 b22 b33 b12 b13 b23] = advs(sx2, sy2, sz-z(iTri+1), z(iTri+1), beta, pr, lss, lds, lts);

      % Rotate tensors to correct for strike
      bxx                       = a11-b11;
      byy                       = a22-b22;
      bzz                       = a33-b33;
      bxy                       = a12-b12;
      bxz                       = a13-b13;
      byz                       = a23-b23;
      
      g                         = pi/180*strike;
      e11n                      = (cos(g)*bxx-sin(g)*bxy)*cos(g)-(cos(g)*bxy-sin(g)*byy)*sin(g);
      e12n                      = (cos(g)*bxx-sin(g)*bxy)*sin(g)+(cos(g)*bxy-sin(g)*byy)*cos(g);
      e13n                      = cos(g)*bxz-sin(g)*byz;
      e22n                      = (sin(g)*bxx+cos(g)*bxy)*sin(g)+(sin(g)*bxy+cos(g)*byy)*cos(g);
      e23n                      = sin(g)*bxz+cos(g)*byz;
      e33n                      = bzz;
      
      % Add the strains from current leg
      xx                        = xx + e11n;
      yy                        = yy + e22n;
      zz                        = zz + e33n;
      xy                        = xy + e12n;
      xz                        = xz + e13n;
      yz                        = yz + e23n;
   end
end


function [a b] = swap(a, b)
% Swap two values
temp                            = a;
a                               = b;
b                               = temp;


function [xp yp] = RotateXyVec(x, y, alpha)
% Rotate a vector by an angle alpha
x                             = x(:);
y                             = y(:);
alpha                         = pi/180*alpha;
xp                            = cos(alpha).*x - sin(alpha).*y;
yp                            = sin(alpha).*x + cos(alpha).*y;


function [e11 e22 e33 e12 e13 e23] = advs(y1, y2, y3, a, b, nu, B1, B2, B3)
% These are the strains in a uniform elastic half space due to slip
% on an angular dislocation.  They were calculated by symbolically
% differentiating the expressions for the displacements (Comninou and
% Dunders, 1975, with typos noted by Thomas 1993) then combining the
% elements of the displacement gradient tensor to form the strain tensor.

sinb = sin(b);
cosb = cos(b);
cotb = 1./tan(b);

y12 = y1.^2;
y22 = y2.^2;
y32 = y3.^2;
y122 = y1.^2 + y2.^2;

oh = 1./2;
th = 3./2;
oe = 1./8;

sys = y12+y22+y32;
sysq = sys.^(oh);


nunu = 2.*nu;
aa = 2.*a;
y3aa = y3+2.*a;
y3aa2 = (y3+2.*a).^2;
yy2 = y122+y3aa2;
yy2q = yy2.^(oh);


e11 = zeros(size(y1));
e22 = e11; 
e33 = e11; 
e12 = e11; 
e13 = e11; 
e23 = e11; 

e11 = B1.*(oe.*((2-nunu).*(2.*y2./y12./(1+y22./y12)-y2./(y1.*cosb-y3.*sinb).^2.*cosb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y1-y2.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb-y3.*sinb))./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))-y2.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa))-y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y1-1./(sys)./(sysq-y3).^2.*y1-1./yy2.^(th)./(yy2q+y3+aa).*y1-1./yy2./(yy2q+y3+aa).^2.*y1)-y2.*cosb.*((1./sysq.*sinb.*y1-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)+(1./yy2q.*sinb.*y1-1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb)))./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*(y2./y12./(1+y22./y12)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2-(1-nunu).*y2./(yy2q+y3+aa).^2.*((1-nunu-a./yy2q).*cotb-y1./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y1+(1-nunu).*y2./(yy2q+y3+aa).*(a./yy2.^(th).*y1.*cotb-1./(yy2q+y3+aa).*(nu+a./yy2q)+y12./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q+y12./(yy2q+y3+aa).*a./yy2.^(th))-(1-nunu).*y2.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(1-nunu).*y2.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-3.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*y1-y2.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2).*y1-y2.*(y3+a)./yy2./(yy2q+y3+aa).^2.*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2).*y1+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(1./(yy2q+y3+aa).*(nunu+a./yy2q)-y12./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q-y12./(yy2q+y3+aa).*a./yy2.^(th)+a./yy2-aa.*y12./yy2.^2)-y2.*(y3+a)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2).*y1-y2.*(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2).*(1./yy2q.*y1-sinb)+y2.*(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb).*(1./yy2q.*y1-sinb)+cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y1.*((1-nunu).*cosb-a./yy2q).*cotb+(yy2q.*cosb+y3+aa).*a./yy2.^(th).*y1.*cotb+(2-nunu).*(1./yy2q.*sinb.*y1-1).*cosb)+aa.*y3aa.*cosb.*cotb./yy2.^2.*y1))./pi./(1-nu));
e11 = e11 + B2.*(oe.*((-1+nunu).*(1./sysq.*y1./(sysq-y3)+1./yy2q.*y1./(yy2q+y3+aa)-cosb.*((1./sysq.*y1-sinb)./(sysq-y1.*sinb-y3.*cosb)+(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb)))+2.*y1.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa))+y12.*(-1./(sys).^(th)./(sysq-y3).*y1-1./(sys)./(sysq-y3).^2.*y1-1./yy2.^(th)./(yy2q+y3+aa).*y1-1./yy2./(yy2q+y3+aa).^2.*y1)+cosb.*(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb)+(y1.*cosb-y3.*sinb).*(1./sysq.*sinb.*y1-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)+cosb.*(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+(y1.*cosb+y3aa.*sinb).*(1./yy2q.*sinb.*y1-1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2+nu)./yy2q.*y1./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1).*cosb.*(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb))-(1-nunu)./(yy2q+y3+aa).^2.*((-1+nunu).*y1.*cotb+nu.*y3aa-a+a.*y1.*cotb./yy2q+y12./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y1+(1-nunu)./(yy2q+y3+aa).*((-1+nunu).*cotb+a.*cotb./yy2q-a.*y12.*cotb./yy2.^(th)+2.*y1./(yy2q+y3+aa).*(nu+a./yy2q)-y1.^3./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q-y1.^3./(yy2q+y3+aa).*a./yy2.^(th))+(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((y1.*cosb+y3aa.*sinb).*cosb-a.*(yy2q.*sinb-y1)./yy2q./cosb).*(1./yy2q.*y1-sinb)-(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb.^2-a.*(1./yy2q.*sinb.*y1-1)./yy2q./cosb+a.*(yy2q.*sinb-y1)./yy2.^(th)./cosb.*y1)-a.*(y3+a).*cotb./yy2.^(th)+3.*a.*y12.*(y3+a).*cotb./yy2.^(5./2)-(y3+a)./(yy2q+y3+aa).^2.*(nunu+1./yy2q.*((1-nunu).*y1.*cotb+a)-y12./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)-a.*y12./yy2.^(th))./yy2q.*y1+(y3+a)./(yy2q+y3+aa).*(-1./yy2.^(th).*((1-nunu).*y1.*cotb+a).*y1+1./yy2q.*(1-nunu).*cotb-2.*y1./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)+y1.^3./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q)+y1.^3./yy2./(yy2q+y3+aa).^2.*(nunu+a./yy2q)+y1.^3./yy2.^2./(yy2q+y3+aa).*a-aa./yy2.^(th).*y1+3.*a.*y1.^3./yy2.^(5./2))-(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(-cosb.*sinb+a.*y1.*y3aa./yy2.^(th)./cosb+(yy2q.*sinb-y1)./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb))).*(1./yy2q.*y1-sinb)+(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(a.*y3aa./yy2.^(th)./cosb-3.*a.*y12.*y3aa./yy2.^(5./2)./cosb+(1./yy2q.*sinb.*y1-1)./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb))-(yy2q.*sinb-y1)./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)).*y1+(yy2q.*sinb-y1)./yy2q.*(-1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(1./yy2q.*y1-sinb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*y1)))./pi./(1-nu));
e11 = e11 + B3.*(oe.*y2.*sinb.*((1./sysq.*sinb.*y1-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)+(1./yy2q.*sinb.*y1-1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))./pi./(1-nu)+1./4.*((1-nunu).*(-y2./(yy2q+y3+aa).^2.*(1+a./yy2q)./yy2q.*y1-y2./(yy2q+y3+aa).*a./yy2.^(th).*y1+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1)+y2.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa)).*y1-y2.*(y3+a)./yy2q.*(-aa./yy2.^2.*y1-1./(yy2q+y3+aa).^2./yy2q.*y1)-y2.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*y1-y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(1./yy2q.*y1-sinb)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-aa.*y3aa./yy2.^2.*y1))./pi./(1-nu));

e22 = B1.*(oe.*((1-nunu).*(1./sysq.*y2./(sysq-y3)+1./yy2q.*y2./(yy2q+y3+aa)-cosb.*(1./sysq.*y2./(sysq-y1.*sinb-y3.*cosb)+1./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb)))-2.*y2.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa)-cosb.*(1./sysq./(sysq-y1.*sinb-y3.*cosb)+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)))-y22.*(-1./(sys).^(th)./(sysq-y3).*y2-1./(sys)./(sysq-y3).^2.*y2-1./yy2.^(th)./(yy2q+y3+aa).*y2-1./yy2./(yy2q+y3+aa).^2.*y2-cosb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-1./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-1./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2)))./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2-nu)./yy2q.*y2./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1-nunu).*cosb./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb))+(1-nunu)./(yy2q+y3+aa).^2.*(y1.*cotb.*(1-nunu-a./yy2q)+nu.*y3aa-a+y22./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y2-(1-nunu)./(yy2q+y3+aa).*(a.*y1.*cotb./yy2.^(th).*y2+2.*y2./(yy2q+y3+aa).*(nu+a./yy2q)-y2.^3./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q-y2.^3./(yy2q+y3+aa).*a./yy2.^(th))+(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2+(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2+3.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*y1-(y3+a)./(yy2q+y3+aa).^2.*(-nunu+1./yy2q.*((1-nunu).*y1.*cotb-a)+y22./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y22./yy2.^(th))./yy2q.*y2+(y3+a)./(yy2q+y3+aa).*(-1./yy2.^(th).*((1-nunu).*y1.*cotb-a).*y2+2.*y2./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)-y2.^3./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q)-y2.^3./yy2./(yy2q+y3+aa).^2.*(nunu+a./yy2q)-y2.^3./yy2.^2./(yy2q+y3+aa).*a+aa./yy2.^(th).*y2-3.*a.*y2.^3./yy2.^(5./2))-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.^2-1./yy2q.*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb)+a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)))./yy2q.*y2+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2.^(th).*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb).*y2-3.*a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(5./2).*y2+1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*y2+1./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*y2-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y2.*cosb.^2+a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th).*(yy2q.*cosb+y3+aa).*y2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.*cosb.*y2)))./pi./(1-nu));
e22 = e22 + B2.*(oe.*((2-nunu).*(-2./y1./(1+y22./y12)+1./(y1.*cosb-y3.*sinb)./(1+y22./(y1.*cosb-y3.*sinb).^2)+(sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)+y22./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)-2.*y22.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+y1.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa))+y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y2-1./(sys)./(sysq-y3).^2.*y2-1./yy2.^(th)./(yy2q+y3+aa).*y2-1./yy2./(yy2q+y3+aa).^2.*y2)-(y1.*cosb-y3.*sinb)./sysq./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb+y3aa.*sinb)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-y2.*(-(y1.*cosb-y3.*sinb)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(y1.*cosb-y3.*sinb)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-(y1.*cosb+y3aa.*sinb)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-(y1.*cosb+y3aa.*sinb)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2))./pi./(1-nu)+1./4.*((2-nunu).*(1-nunu).*(-1./y1./(1+y22./y12)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2+(1-nunu)./(yy2q+y3+aa).*((-1+nunu+a./yy2q).*cotb+y1./(yy2q+y3+aa).*(nu+a./yy2q))-(1-nunu).*y22./(yy2q+y3+aa).^2.*((-1+nunu+a./yy2q).*cotb+y1./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q+(1-nunu).*y2./(yy2q+y3+aa).*(-a./yy2.^(th).*y2.*cotb-y1./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q.*y2-y2./(yy2q+y3+aa).*a./yy2.^(th).*y1)-(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+(1-nunu).*y22.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb)./yy2q+(1-nunu).*y22.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb-a.*(y3+a).*cotb./yy2.^(th)+3.*a.*y22.*(y3+a).*cotb./yy2.^(5./2)+(y3+a)./yy2q./(yy2q+y3+aa).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa)))-y22.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa)))-y22.*(y3+a)./yy2./(yy2q+y3+aa).^2.*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa)))+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(nunu.*y1./(yy2q+y3+aa).^2./yy2q.*y2+a.*y1./yy2.^(th).*(1./yy2q+1./(yy2q+y3+aa)).*y2-a.*y1./yy2q.*(-1./yy2.^(th).*y2-1./(yy2q+y3+aa).^2./yy2q.*y2))+(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb)-y22.*(y3+a).*cotb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb)-y22.*(y3+a).*cotb./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb)+y2.*(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb)./yy2q.*y2-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*y2-aa.*y3aa./yy2.^2./cosb.*y2))./pi./(1-nu));
e22 = e22 + B3.*(oe.*((1-nunu).*sinb.*(1./sysq.*y2./(sysq-y1.*sinb-y3.*cosb)+1./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb))-2.*y2.*sinb.*(1./sysq./(sysq-y1.*sinb-y3.*cosb)+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))-y22.*sinb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-1./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-1./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2))./pi./(1-nu)+1./4.*((1-nunu).*(-sinb./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+y2./(yy2q+y3+aa).^2.*(1+a./yy2q)./yy2q.*y1+y2./(yy2q+y3+aa).*a./yy2.^(th).*y1-(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2-(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2)-y2.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa)).*y1+y1.*(y3+a)./yy2q.*(-aa./yy2.^2.*y2-1./(yy2q+y3+aa).^2./yy2q.*y2)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(sinb.*(cosb-a./yy2q)+(y1.*cosb+y3aa.*sinb)./yy2q.*(1+a.*y3aa./yy2)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)))./yy2q.*y2-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(sinb.*a./yy2.^(th).*y2-(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(1+a.*y3aa./yy2).*y2-2.*(y1.*cosb+y3aa.*sinb)./yy2.^(5./2).*a.*y3aa.*y2+1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*y2+1./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*y2-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y2.*cosb.*sinb+a.*(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(yy2q.*cosb+y3+aa).*y2-a.*(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y2)))./pi./(1-nu));

e33 = B1.*(oe.*y2.*(-1./(sys).^(th).*y3+oh./yy2.^(th).*(2.*y3+4.*a)-cosb.*((1./sysq.*cosb.*y3-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)-(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+oh.*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)+(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb)))./pi./(1-nu)+1./4.*((2-nunu).*((1-nunu).*(-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb-y2./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y2./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a)+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a))+y2./yy2q.*(nunu./(yy2q+y3+aa)+a./yy2)-oh.*y2.*(y3+a)./yy2.^(th).*(nunu./(yy2q+y3+aa)+a./yy2).*(2.*y3+4.*a)+y2.*(y3+a)./yy2q.*(-nunu./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1)-a./yy2.^2.*(2.*y3+4.*a))+y2.*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2)-oh.*y2.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2).*(2.*y3+4.*a)-y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)-a./yy2+a.*y3aa./yy2.^2.*(2.*y3+4.*a)))./pi./(1-nu));
e33 = e33 + B2.*(oe.*((-1+nunu).*sinb.*((1./sysq.*y3-cosb)./(sysq-y1.*sinb-y3.*cosb)-(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb))-y1.*(-1./(sys).^(th).*y3+oh./yy2.^(th).*(2.*y3+4.*a))-sinb.*(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb)+(y1.*cosb-y3.*sinb).*(1./sysq.*cosb.*y3-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)-sinb.*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb).*(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+oh.*(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*cotb.*((oh./yy2q.*(2.*y3+4.*a)+1)./(yy2q+y3+aa)-cosb.*(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb))+(2-nunu).*y1./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)+oh.*(2-nunu).*y1./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a)+(2-nunu).*sinb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)+1./yy2q.*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2)-oh.*(y3+a)./yy2.^(th).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2).*(2.*y3+4.*a)+(y3+a)./yy2q.*(nunu.*y1./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1)+a.*y1./yy2.^2.*(2.*y3+4.*a))-1./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb.*sinb+(yy2q.*cosb+y3+aa).*cotb./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))+a./yy2q.*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)))+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.*sinb+(yy2q.*cosb+y3+aa).*cotb./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))+a./yy2q.*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*((oh./yy2q.*cosb.*(2.*y3+4.*a)+1).*cotb./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))-oh.*(yy2q.*cosb+y3+aa).*cotb./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb)).*(2.*y3+4.*a)+(yy2q.*cosb+y3+aa).*cotb./yy2q.*(-(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))-oh.*a./yy2.^(th).*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)).*(2.*y3+4.*a)+a./yy2q.*(-(y1.*cosb+y3aa.*sinb)./yy2-y3aa.*sinb./yy2+y3aa.*(y1.*cosb+y3aa.*sinb)./yy2.^2.*(2.*y3+4.*a)-sinb.*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb).*(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+oh.*(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))))./pi./(1-nu));
e33 = e33 + B3.*(oe.*((2-nunu).*(y2./(y1.*cosb-y3.*sinb).^2.*sinb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y3+y2.*sysq.*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)+y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)-(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+y2.*sinb.*((1./sysq.*cosb.*y3-1)./sysq./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)-(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+oh.*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)+(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb)))./pi./(1-nu)+1./4.*((2-nunu).*(-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))-(2-nunu).*y2.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(2-nunu).*y2.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)+y2.*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-oh.*y2.*(y3+a).*sinb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(2.*y3+4.*a)-y2.*(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+y2.*(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)+a./yy2-a.*y3aa./yy2.^2.*(2.*y3+4.*a)))./pi./(1-nu));

e12 = oh.*B1.*(oe.*((2-nunu).*(-2./y1./(1+y22./y12)+1./(y1.*cosb-y3.*sinb)./(1+y22./(y1.*cosb-y3.*sinb).^2)+(sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)+y22./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)-2.*y22.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))-y1.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa))-y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y2-1./(sys)./(sysq-y3).^2.*y2-1./yy2.^(th)./(yy2q+y3+aa).*y2-1./yy2./(yy2q+y3+aa).^2.*y2)-cosb.*((sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb)+(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))-y2.*cosb.*(1./(sys).*sinb.*y2./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(sysq.*sinb-y1)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2+1./yy2.*sinb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)-(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-(yy2q.*sinb-y1)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2))./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*(-1./y1./(1+y22./y12)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2+(1-nunu)./(yy2q+y3+aa).*((1-nunu-a./yy2q).*cotb-y1./(yy2q+y3+aa).*(nu+a./yy2q))-(1-nunu).*y22./(yy2q+y3+aa).^2.*((1-nunu-a./yy2q).*cotb-y1./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q+(1-nunu).*y2./(yy2q+y3+aa).*(a./yy2.^(th).*y2.*cotb+y1./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q.*y2+y2./(yy2q+y3+aa).*a./yy2.^(th).*y1)+(1-nunu).*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(1-nunu).*y22.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q-(1-nunu).*y22.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)+a.*(y3+a).*cotb./yy2.^(th)-3.*a.*y22.*(y3+a).*cotb./yy2.^(5./2)+(y3+a)./yy2q./(yy2q+y3+aa).*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2)-y22.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2)-y22.*(y3+a)./yy2./(yy2q+y3+aa).^2.*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2)+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(-y1./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q.*y2-y2./(yy2q+y3+aa).*a./yy2.^(th).*y1-aa.*y1./yy2.^2.*y2)+(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2)-y22.*(y3+a)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2)-y22.*(y3+a)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2)+y2.*(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)./yy2q.*y2+cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y2.*((1-nunu).*cosb-a./yy2q).*cotb+(yy2q.*cosb+y3+aa).*a./yy2.^(th).*y2.*cotb+(2-nunu)./yy2q.*sinb.*y2.*cosb)+aa.*y3aa.*cosb.*cotb./yy2.^2.*y2))./pi./(1-nu));
e12 = e12 + oh.*B2.*(oe.*((-1+nunu).*(1./sysq.*y2./(sysq-y3)+1./yy2q.*y2./(yy2q+y3+aa)-cosb.*(1./sysq.*y2./(sysq-y1.*sinb-y3.*cosb)+1./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb)))+y12.*(-1./(sys).^(th)./(sysq-y3).*y2-1./(sys)./(sysq-y3).^2.*y2-1./yy2.^(th)./(yy2q+y3+aa).*y2-1./yy2./(yy2q+y3+aa).^2.*y2)+(y1.*cosb-y3.*sinb)./(sys).*sinb.*y2./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2+(y1.*cosb+y3aa.*sinb)./yy2.*sinb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2)./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2+nu)./yy2q.*y2./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1).*cosb./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb))-(1-nunu)./(yy2q+y3+aa).^2.*((-1+nunu).*y1.*cotb+nu.*y3aa-a+a.*y1.*cotb./yy2q+y12./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y2+(1-nunu)./(yy2q+y3+aa).*(-a.*y1.*cotb./yy2.^(th).*y2-y12./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q.*y2-y12./(yy2q+y3+aa).*a./yy2.^(th).*y2)+(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((y1.*cosb+y3aa.*sinb).*cosb-a.*(yy2q.*sinb-y1)./yy2q./cosb)./yy2q.*y2-(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(-a./yy2.*sinb.*y2./cosb+a.*(yy2q.*sinb-y1)./yy2.^(th)./cosb.*y2)+3.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*y1-(y3+a)./(yy2q+y3+aa).^2.*(nunu+1./yy2q.*((1-nunu).*y1.*cotb+a)-y12./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)-a.*y12./yy2.^(th))./yy2q.*y2+(y3+a)./(yy2q+y3+aa).*(-1./yy2.^(th).*((1-nunu).*y1.*cotb+a).*y2+y12./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q).*y2+y12./yy2./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*y2+y12./yy2.^2./(yy2q+y3+aa).*a.*y2+3.*a.*y12./yy2.^(5./2).*y2)-(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(-cosb.*sinb+a.*y1.*y3aa./yy2.^(th)./cosb+(yy2q.*sinb-y1)./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)))./yy2q.*y2+(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(-3.*a.*y1.*y3aa./yy2.^(5./2)./cosb.*y2+1./yy2.*sinb.*y2.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb))-(yy2q.*sinb-y1)./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)).*y2+(yy2q.*sinb-y1)./yy2q.*(-1./yy2q.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb)./yy2q.*y2+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*y2)))./pi./(1-nu));
e12 = e12 + oh.*B3.*(oe.*sinb.*((sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb)+(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))./pi./(1-nu)+oe.*y2.*sinb.*(1./(sys).*sinb.*y2./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(sysq.*sinb-y1)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2+1./yy2.*sinb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)-(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2-(yy2q.*sinb-y1)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2)./pi./(1-nu)+1./4.*((1-nunu).*(1./(yy2q+y3+aa).*(1+a./yy2q)-y22./(yy2q+y3+aa).^2.*(1+a./yy2q)./yy2q-y22./(yy2q+y3+aa).*a./yy2.^(th)-cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+y22.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q+y22.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th))-(y3+a)./yy2q.*(a./yy2+1./(yy2q+y3+aa))+y22.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa))-y2.*(y3+a)./yy2q.*(-aa./yy2.^2.*y2-1./(yy2q+y3+aa).^2./yy2q.*y2)+(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-y22.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-y22.*(y3+a).*cosb./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2-aa.*y3aa./yy2.^2.*y2))./pi./(1-nu))+oh.*B1.*(oe.*((1-nunu).*(1./sysq.*y1./(sysq-y3)+1./yy2q.*y1./(yy2q+y3+aa)-cosb.*((1./sysq.*y1-sinb)./(sysq-y1.*sinb-y3.*cosb)+(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb)))-y22.*(-1./(sys).^(th)./(sysq-y3).*y1-1./(sys)./(sysq-y3).^2.*y1-1./yy2.^(th)./(yy2q+y3+aa).*y1-1./yy2./(yy2q+y3+aa).^2.*y1-cosb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-1./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)-1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))))./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2-nu)./yy2q.*y1./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1-nunu).*cosb.*(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb))+(1-nunu)./(yy2q+y3+aa).^2.*(y1.*cotb.*(1-nunu-a./yy2q)+nu.*y3aa-a+y22./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y1-(1-nunu)./(yy2q+y3+aa).*((1-nunu-a./yy2q).*cotb+a.*y12.*cotb./yy2.^(th)-y22./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q.*y1-y22./(yy2q+y3+aa).*a./yy2.^(th).*y1)-(1-nunu).*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)+(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-a.*(y3+a).*cotb./yy2.^(th)+3.*a.*y12.*(y3+a).*cotb./yy2.^(5./2)-(y3+a)./(yy2q+y3+aa).^2.*(-nunu+1./yy2q.*((1-nunu).*y1.*cotb-a)+y22./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y22./yy2.^(th))./yy2q.*y1+(y3+a)./(yy2q+y3+aa).*(-1./yy2.^(th).*((1-nunu).*y1.*cotb-a).*y1+1./yy2q.*(1-nunu).*cotb-y22./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q).*y1-y22./yy2./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*y1-y22./yy2.^2./(yy2q+y3+aa).*a.*y1-3.*a.*y22./yy2.^(5./2).*y1)-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.^2-1./yy2q.*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb)+a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa))).*(1./yy2q.*y1-sinb)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2.^(th).*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb).*y1-1./yy2q.*(1-nunu).*cosb.*cotb+a.*y3aa.*cosb.*cotb./yy2.^(th)-3.*a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(5./2).*y1+1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*y1+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*(1./yy2q.*y1-sinb)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-a.*cosb.*cotb./yy2q.*(yy2q.*cosb+y3+aa)+a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th).*(yy2q.*cosb+y3+aa).*y1-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.*cosb.*y1)))./pi./(1-nu))+oh.*B2.*(oe.*((2-nunu).*(2.*y2./y12./(1+y22./y12)-y2./(y1.*cosb-y3.*sinb).^2.*cosb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y1-y2.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb-y3.*sinb))./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+y2.*(1./sysq./(sysq-y3)+1./yy2q./(yy2q+y3+aa))+y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y1-1./(sys)./(sysq-y3).^2.*y1-1./yy2.^(th)./(yy2q+y3+aa).*y1-1./yy2./(yy2q+y3+aa).^2.*y1)-y2.*(cosb./sysq./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(y1.*cosb-y3.*sinb)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)+cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-(y1.*cosb+y3aa.*sinb)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb)))./pi./(1-nu)+1./4.*((2-nunu).*(1-nunu).*(y2./y12./(1+y22./y12)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2-(1-nunu).*y2./(yy2q+y3+aa).^2.*((-1+nunu+a./yy2q).*cotb+y1./(yy2q+y3+aa).*(nu+a./yy2q))./yy2q.*y1+(1-nunu).*y2./(yy2q+y3+aa).*(-a./yy2.^(th).*y1.*cotb+1./(yy2q+y3+aa).*(nu+a./yy2q)-y12./(yy2q+y3+aa).^2.*(nu+a./yy2q)./yy2q-y12./(yy2q+y3+aa).*a./yy2.^(th))+(1-nunu).*y2.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(1./yy2q.*y1-sinb)+(1-nunu).*y2.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*y1+3.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*y1-y2.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa))).*y1-y2.*(y3+a)./yy2./(yy2q+y3+aa).^2.*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa))).*y1+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(-nunu./(yy2q+y3+aa)+nunu.*y12./(yy2q+y3+aa).^2./yy2q-a./yy2q.*(1./yy2q+1./(yy2q+y3+aa))+a.*y12./yy2.^(th).*(1./yy2q+1./(yy2q+y3+aa))-a.*y1./yy2q.*(-1./yy2.^(th).*y1-1./(yy2q+y3+aa).^2./yy2q.*y1))-y2.*(y3+a).*cotb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb).*y1-y2.*(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb).*(1./yy2q.*y1-sinb)+y2.*(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(1./yy2q.*y1-sinb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*y1-aa.*y3aa./yy2.^2./cosb.*y1))./pi./(1-nu))+oh.*B3.*(oe.*((1-nunu).*sinb.*((1./sysq.*y1-sinb)./(sysq-y1.*sinb-y3.*cosb)+(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb))-y22.*sinb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-1./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)-1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb)))./pi./(1-nu)+1./4.*((1-nunu).*(-sinb.*(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb)-1./(yy2q+y3+aa).*(1+a./yy2q)+y12./(yy2q+y3+aa).^2.*(1+a./yy2q)./yy2q+y12./(yy2q+y3+aa).*a./yy2.^(th)+cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1)+(y3+a)./yy2q.*(a./yy2+1./(yy2q+y3+aa))-y12.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa))+y1.*(y3+a)./yy2q.*(-aa./yy2.^2.*y1-1./(yy2q+y3+aa).^2./yy2q.*y1)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(sinb.*(cosb-a./yy2q)+(y1.*cosb+y3aa.*sinb)./yy2q.*(1+a.*y3aa./yy2)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa))).*(1./yy2q.*y1-sinb)-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(sinb.*a./yy2.^(th).*y1+cosb./yy2q.*(1+a.*y3aa./yy2)-(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(1+a.*y3aa./yy2).*y1-2.*(y1.*cosb+y3aa.*sinb)./yy2.^(5./2).*a.*y3aa.*y1+1./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*y1+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*(1./yy2q.*y1-sinb)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-a.*cosb./yy2q.*(yy2q.*cosb+y3+aa)+a.*(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(yy2q.*cosb+y3+aa).*y1-a.*(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y1)))./pi./(1-nu));

e13 = oh.*B1.*(oe.*((2-nunu).*(y2./(y1.*cosb-y3.*sinb).^2.*sinb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y3+y2.*sysq.*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))-y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y3-1./sysq./(sysq-y3).^2.*(1./sysq.*y3-1)-oh./yy2.^(th)./(yy2q+y3+aa).*(2.*y3+4.*a)-1./yy2q./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1))-y2.*cosb.*(1./(sys).*sinb.*y3./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)+oh./yy2.*sinb.*(2.*y3+4.*a)./(yy2q-y1.*sinb+y3aa.*cosb)-oh.*(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb)))./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*(-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2-(1-nunu).*y2./(yy2q+y3+aa).^2.*((1-nunu-a./yy2q).*cotb-y1./(yy2q+y3+aa).*(nu+a./yy2q)).*(oh./yy2q.*(2.*y3+4.*a)+1)+(1-nunu).*y2./(yy2q+y3+aa).*(oh.*a./yy2.^(th).*(2.*y3+4.*a).*cotb+y1./(yy2q+y3+aa).^2.*(nu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)+oh.*y1./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a))-(1-nunu).*y2.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(1-nunu).*y2.*cosb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)+a./yy2.^(th).*y2.*cotb-th.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*(2.*y3+4.*a)+y2./yy2q./(yy2q+y3+aa).*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2)-oh.*y2.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2).*(2.*y3+4.*a)-y2.*(y3+a)./yy2q./(yy2q+y3+aa).^2.*((-1+nunu).*cotb+y1./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y1./yy2).*(oh./yy2q.*(2.*y3+4.*a)+1)+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(-y1./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y1./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a)-a.*y1./yy2.^2.*(2.*y3+4.*a))+y2./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2)-oh.*y2.*(y3+a)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2).*(2.*y3+4.*a)-y2.*(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb)-a.*y3aa.*cosb.*cotb./yy2).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+y2.*(y3+a)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa).*((1-nunu).*cosb-a./yy2q).*cotb+(2-nunu).*(yy2q.*sinb-y1).*cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+cosb./(yy2q-y1.*sinb+y3aa.*cosb).*((oh./yy2q.*cosb.*(2.*y3+4.*a)+1).*((1-nunu).*cosb-a./yy2q).*cotb+oh.*(yy2q.*cosb+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a).*cotb+oh.*(2-nunu)./yy2q.*sinb.*(2.*y3+4.*a).*cosb)-a.*cosb.*cotb./yy2+a.*y3aa.*cosb.*cotb./yy2.^2.*(2.*y3+4.*a)))./pi./(1-nu));
e13 = e13 + oh.*B2.*(oe.*((-1+nunu).*((1./sysq.*y3-1)./(sysq-y3)+(oh./yy2q.*(2.*y3+4.*a)+1)./(yy2q+y3+aa)-cosb.*((1./sysq.*y3-cosb)./(sysq-y1.*sinb-y3.*cosb)+(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb)))+y12.*(-1./(sys).^(th)./(sysq-y3).*y3-1./sysq./(sysq-y3).^2.*(1./sysq.*y3-1)-oh./yy2.^(th)./(yy2q+y3+aa).*(2.*y3+4.*a)-1./yy2q./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1))-sinb.*(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb)+(y1.*cosb-y3.*sinb)./(sys).*sinb.*y3./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(y1.*cosb-y3.*sinb).*(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)+sinb.*(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)+oh.*(y1.*cosb+y3aa.*sinb)./yy2.*sinb.*(2.*y3+4.*a)./(yy2q-y1.*sinb+y3aa.*cosb)-oh.*(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-(y1.*cosb+y3aa.*sinb).*(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2+nu).*(oh./yy2q.*(2.*y3+4.*a)+1)./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1).*cosb.*(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb))-(1-nunu)./(yy2q+y3+aa).^2.*((-1+nunu).*y1.*cotb+nu.*y3aa-a+a.*y1.*cotb./yy2q+y12./(yy2q+y3+aa).*(nu+a./yy2q)).*(oh./yy2q.*(2.*y3+4.*a)+1)+(1-nunu)./(yy2q+y3+aa).*(nu-oh.*a.*y1.*cotb./yy2.^(th).*(2.*y3+4.*a)-y12./(yy2q+y3+aa).^2.*(nu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y12./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a))+(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((y1.*cosb+y3aa.*sinb).*cosb-a.*(yy2q.*sinb-y1)./yy2q./cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-(1-nunu).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb.*sinb-oh.*a./yy2.*sinb.*(2.*y3+4.*a)./cosb+oh.*a.*(yy2q.*sinb-y1)./yy2.^(th)./cosb.*(2.*y3+4.*a))-a./yy2.^(th).*y1.*cotb+th.*a.*y1.*(y3+a).*cotb./yy2.^(5./2).*(2.*y3+4.*a)+1./(yy2q+y3+aa).*(nunu+1./yy2q.*((1-nunu).*y1.*cotb+a)-y12./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)-a.*y12./yy2.^(th))-(y3+a)./(yy2q+y3+aa).^2.*(nunu+1./yy2q.*((1-nunu).*y1.*cotb+a)-y12./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)-a.*y12./yy2.^(th)).*(oh./yy2q.*(2.*y3+4.*a)+1)+(y3+a)./(yy2q+y3+aa).*(-oh./yy2.^(th).*((1-nunu).*y1.*cotb+a).*(2.*y3+4.*a)+oh.*y12./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q).*(2.*y3+4.*a)+y12./yy2q./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)+oh.*y12./yy2.^2./(yy2q+y3+aa).*a.*(2.*y3+4.*a)+th.*a.*y12./yy2.^(5./2).*(2.*y3+4.*a))+cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(-cosb.*sinb+a.*y1.*y3aa./yy2.^(th)./cosb+(yy2q.*sinb-y1)./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)))-(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(-cosb.*sinb+a.*y1.*y3aa./yy2.^(th)./cosb+(yy2q.*sinb-y1)./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb))).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+(y3+a).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(a./yy2.^(th)./cosb.*y1-th.*a.*y1.*y3aa./yy2.^(5./2)./cosb.*(2.*y3+4.*a)+oh./yy2.*sinb.*(2.*y3+4.*a).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb))-oh.*(yy2q.*sinb-y1)./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)).*(2.*y3+4.*a)+(yy2q.*sinb-y1)./yy2q.*(-(oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*(2.*y3+4.*a))))./pi./(1-nu));
e13 = e13 + oh.*B3.*(oe.*y2.*sinb.*(1./(sys).*sinb.*y3./(sysq-y1.*sinb-y3.*cosb)-(sysq.*sinb-y1)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(sysq.*sinb-y1)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)+oh./yy2.*sinb.*(2.*y3+4.*a)./(yy2q-y1.*sinb+y3aa.*cosb)-oh.*(yy2q.*sinb-y1)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-(yy2q.*sinb-y1)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))./pi./(1-nu)+1./4.*((1-nunu).*(-y2./(yy2q+y3+aa).^2.*(1+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y2./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a)+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a))-y2./yy2q.*(a./yy2+1./(yy2q+y3+aa))+oh.*y2.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa)).*(2.*y3+4.*a)-y2.*(y3+a)./yy2q.*(-a./yy2.^2.*(2.*y3+4.*a)-1./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1))+y2.*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-oh.*y2.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(2.*y3+4.*a)-y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)+a./yy2-a.*y3aa./yy2.^2.*(2.*y3+4.*a)))./pi./(1-nu))+oh.*B1.*(oe.*y2.*(-1./(sys).^(th).*y1+1./yy2.^(th).*y1-cosb.*(1./(sys).*cosb.*y1./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)-1./yy2.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1+(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb)))./pi./(1-nu)+1./4.*((2-nunu).*((1-nunu).*(y2./y12./(1+y22./y12)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb-y1./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q.*y2-y2./(yy2q+y3+aa).*a./yy2.^(th).*y1+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)+y2.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1)-y2.*(y3+a)./yy2.^(th).*(nunu./(yy2q+y3+aa)+a./yy2).*y1+y2.*(y3+a)./yy2q.*(-nunu./(yy2q+y3+aa).^2./yy2q.*y1-aa./yy2.^2.*y1)-y2.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2).*y1-y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2).*(1./yy2q.*y1-sinb)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1+aa.*y3aa./yy2.^2.*y1))./pi./(1-nu))+oh.*B2.*(oe.*((-1+nunu).*sinb.*((1./sysq.*y1-sinb)./(sysq-y1.*sinb-y3.*cosb)-(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb))-1./sysq+1./yy2q-y1.*(-1./(sys).^(th).*y1+1./yy2.^(th).*y1)+cosb.*(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb)+(y1.*cosb-y3.*sinb)./(sys).*cosb.*y1./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)-cosb.*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*cotb.*(1./yy2q.*y1./(yy2q+y3+aa)-cosb.*(1./yy2q.*y1-sinb)./(yy2q-y1.*sinb+y3aa.*cosb))-(2-nunu)./(yy2q+y3+aa).*(nunu+a./yy2q)+(2-nunu).*y12./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q+(2-nunu).*y12./(yy2q+y3+aa).*a./yy2.^(th)+(2-nunu).*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-(y3+a)./yy2.^(th).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2).*y1+(y3+a)./yy2q.*(-nunu./(yy2q+y3+aa)+nunu.*y12./(yy2q+y3+aa).^2./yy2q-a./yy2+aa.*y12./yy2.^2)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.*sinb+(yy2q.*cosb+y3+aa).*cotb./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))+a./yy2q.*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))).*(1./yy2q.*y1-sinb)-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2.*cosb.*y1.*cotb.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))-(yy2q.*cosb+y3+aa).*cotb./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb)).*y1+(yy2q.*cosb+y3+aa).*cotb./yy2q.*(-1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))-a./yy2.^(th).*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)).*y1+a./yy2q.*(-y3aa.*cosb./yy2+2.*y3aa.*(y1.*cosb+y3aa.*sinb)./yy2.^2.*y1-cosb.*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb))))./pi./(1-nu))+oh.*B3.*(oe.*((2-nunu).*(-y2./(y1.*cosb-y3.*sinb).^2.*cosb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y1-y2.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb-y3.*sinb))./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)+y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)-(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+y2.*sinb.*(1./(sys).*cosb.*y1./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y1-(sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y1-sinb)-1./yy2.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y1+(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1./yy2q.*y1-sinb)))./pi./(1-nu)+1./4.*((2-nunu).*(y2./y12./(1+y22./y12)-y2./(y1.*cosb+y3aa.*sinb).^2.*cosb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*y1-y2.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*(2.*y1.*cosb+y3aa.*sinb))./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))-(2-nunu).*y2.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(2-nunu).*y2.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-y2.*(y3+a).*sinb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*y1-y2.*(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2).*(1./yy2q.*y1-sinb)+y2.*(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y1./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(1./yy2q.*y1-sinb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y1-aa.*y3aa./yy2.^2.*y1))./pi./(1-nu));

e23 = oh.*B1.*(oe.*((1-nunu).*((1./sysq.*y3-1)./(sysq-y3)+(oh./yy2q.*(2.*y3+4.*a)+1)./(yy2q+y3+aa)-cosb.*((1./sysq.*y3-cosb)./(sysq-y1.*sinb-y3.*cosb)+(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb)))-y22.*(-1./(sys).^(th)./(sysq-y3).*y3-1./sysq./(sysq-y3).^2.*(1./sysq.*y3-1)-oh./yy2.^(th)./(yy2q+y3+aa).*(2.*y3+4.*a)-1./yy2q./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1)-cosb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-1./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)-oh./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb))))./pi./(1-nu)+1./4.*((1-nunu).*(((2-nunu).*cotb.^2-nu).*(oh./yy2q.*(2.*y3+4.*a)+1)./(yy2q+y3+aa)-((2-nunu).*cotb.^2+1-nunu).*cosb.*(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb))+(1-nunu)./(yy2q+y3+aa).^2.*(y1.*cotb.*(1-nunu-a./yy2q)+nu.*y3aa-a+y22./(yy2q+y3+aa).*(nu+a./yy2q)).*(oh./yy2q.*(2.*y3+4.*a)+1)-(1-nunu)./(yy2q+y3+aa).*(oh.*a.*y1.*cotb./yy2.^(th).*(2.*y3+4.*a)+nu-y22./(yy2q+y3+aa).^2.*(nu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y22./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a))-(1-nunu).*sinb.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*(1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a)-a./yy2.^(th).*y1.*cotb+th.*a.*y1.*(y3+a).*cotb./yy2.^(5./2).*(2.*y3+4.*a)+1./(yy2q+y3+aa).*(-nunu+1./yy2q.*((1-nunu).*y1.*cotb-a)+y22./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y22./yy2.^(th))-(y3+a)./(yy2q+y3+aa).^2.*(-nunu+1./yy2q.*((1-nunu).*y1.*cotb-a)+y22./yy2q./(yy2q+y3+aa).*(nunu+a./yy2q)+a.*y22./yy2.^(th)).*(oh./yy2q.*(2.*y3+4.*a)+1)+(y3+a)./(yy2q+y3+aa).*(-oh./yy2.^(th).*((1-nunu).*y1.*cotb-a).*(2.*y3+4.*a)-oh.*y22./yy2.^(th)./(yy2q+y3+aa).*(nunu+a./yy2q).*(2.*y3+4.*a)-y22./yy2q./(yy2q+y3+aa).^2.*(nunu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y22./yy2.^2./(yy2q+y3+aa).*a.*(2.*y3+4.*a)-th.*a.*y22./yy2.^(5./2).*(2.*y3+4.*a))+1./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb.^2-1./yy2q.*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb)+a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)))-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.^2-1./yy2q.*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb)+a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa))).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(oh./yy2.^(th).*((1-nunu).*(y1.*cosb+y3aa.*sinb).*cotb+a.*cosb).*(2.*y3+4.*a)-1./yy2q.*(1-nunu).*sinb.*cotb+a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th)+a.*y3aa.*sinb.*cotb./yy2.^(th)-th.*a.*y3aa.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(5./2).*(2.*y3+4.*a)+oh./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*(2.*y3+4.*a)+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.^2-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(yy2q.*cosb+y3+aa)).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-a.*sinb.*cotb./yy2q.*(yy2q.*cosb+y3+aa)+oh.*a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2.^(th).*(yy2q.*cosb+y3+aa).*(2.*y3+4.*a)-a.*(y1.*cosb+y3aa.*sinb).*cotb./yy2q.*(oh./yy2q.*cosb.*(2.*y3+4.*a)+1))))./pi./(1-nu));
e23 = e23 + oh.*B2.*(oe.*((2-nunu).*(y2./(y1.*cosb-y3.*sinb).^2.*sinb./(1+y22./(y1.*cosb-y3.*sinb).^2)+(y2./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).*y3+y2.*sysq.*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+y1.*y2.*(-1./(sys).^(th)./(sysq-y3).*y3-1./sysq./(sysq-y3).^2.*(1./sysq.*y3-1)-oh./yy2.^(th)./(yy2q+y3+aa).*(2.*y3+4.*a)-1./yy2q./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1))-y2.*(-sinb./sysq./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-(y1.*cosb-y3.*sinb)./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)+sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)-oh.*(y1.*cosb+y3aa.*sinb)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-(y1.*cosb+y3aa.*sinb)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb)))./pi./(1-nu)+1./4.*((2-nunu).*(1-nunu).*(-y2./(y1.*cosb+y3aa.*sinb).^2.*sinb./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(oh.*y2./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).*(2.*y3+4.*a)-y2.*yy2q.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*y1)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb.^2-(1-nunu).*y2./(yy2q+y3+aa).^2.*((-1+nunu+a./yy2q).*cotb+y1./(yy2q+y3+aa).*(nu+a./yy2q)).*(oh./yy2q.*(2.*y3+4.*a)+1)+(1-nunu).*y2./(yy2q+y3+aa).*(-oh.*a./yy2.^(th).*(2.*y3+4.*a).*cotb-y1./(yy2q+y3+aa).^2.*(nu+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)-oh.*y1./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a))+(1-nunu).*y2.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+oh.*(1-nunu).*y2.*cotb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*(2.*y3+4.*a)-a./yy2.^(th).*y2.*cotb+th.*a.*y2.*(y3+a).*cotb./yy2.^(5./2).*(2.*y3+4.*a)+y2./yy2q./(yy2q+y3+aa).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa)))-oh.*y2.*(y3+a)./yy2.^(th)./(yy2q+y3+aa).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa))).*(2.*y3+4.*a)-y2.*(y3+a)./yy2q./(yy2q+y3+aa).^2.*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2q.*(1./yy2q+1./(yy2q+y3+aa))).*(oh./yy2q.*(2.*y3+4.*a)+1)+y2.*(y3+a)./yy2q./(yy2q+y3+aa).*(nunu.*y1./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1)+oh.*a.*y1./yy2.^(th).*(1./yy2q+1./(yy2q+y3+aa)).*(2.*y3+4.*a)-a.*y1./yy2q.*(-oh./yy2.^(th).*(2.*y3+4.*a)-1./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1)))+y2.*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb)-oh.*y2.*(y3+a).*cotb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb).*(2.*y3+4.*a)-y2.*(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*((-2+nunu).*cosb+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)+a.*y3aa./yy2./cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)+y2.*(y3+a).*cotb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*((oh./yy2q.*cosb.*(2.*y3+4.*a)+1)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+a./yy2q./cosb)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+a./yy2q./cosb).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)./cosb.*(2.*y3+4.*a)+a./yy2./cosb-a.*y3aa./yy2.^2./cosb.*(2.*y3+4.*a)))./pi./(1-nu));
e23 = e23 + oh.*B3.*(oe.*((1-nunu).*sinb.*((1./sysq.*y3-cosb)./(sysq-y1.*sinb-y3.*cosb)+(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb))-y22.*sinb.*(-1./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y3-1./sysq./(sysq-y1.*sinb-y3.*cosb).^2.*(1./sysq.*y3-cosb)-oh./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(2.*y3+4.*a)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(oh./yy2q.*(2.*y3+4.*a)+cosb)))./pi./(1-nu)+1./4.*((1-nunu).*(-sinb.*(oh./yy2q.*(2.*y3+4.*a)+cosb)./(yy2q-y1.*sinb+y3aa.*cosb)+y1./(yy2q+y3+aa).^2.*(1+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+1)+oh.*y1./(yy2q+y3+aa).*a./yy2.^(th).*(2.*y3+4.*a)+sinb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-oh.*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*(2.*y3+4.*a))+y1./yy2q.*(a./yy2+1./(yy2q+y3+aa))-oh.*y1.*(y3+a)./yy2.^(th).*(a./yy2+1./(yy2q+y3+aa)).*(2.*y3+4.*a)+y1.*(y3+a)./yy2q.*(-a./yy2.^2.*(2.*y3+4.*a)-1./(yy2q+y3+aa).^2.*(oh./yy2q.*(2.*y3+4.*a)+1))-1./(yy2q-y1.*sinb+y3aa.*cosb).*(sinb.*(cosb-a./yy2q)+(y1.*cosb+y3aa.*sinb)./yy2q.*(1+a.*y3aa./yy2)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)))+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(sinb.*(cosb-a./yy2q)+(y1.*cosb+y3aa.*sinb)./yy2q.*(1+a.*y3aa./yy2)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa))).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(oh.*sinb.*a./yy2.^(th).*(2.*y3+4.*a)+sinb./yy2q.*(1+a.*y3aa./yy2)-oh.*(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(1+a.*y3aa./yy2).*(2.*y3+4.*a)+(y1.*cosb+y3aa.*sinb)./yy2q.*(a./yy2-a.*y3aa./yy2.^2.*(2.*y3+4.*a))+oh./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*(2.*y3+4.*a)+1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(y22.*cosb.*sinb-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(yy2q.*cosb+y3+aa)).*(oh./yy2q.*(2.*y3+4.*a)+cosb)-1./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-a.*sinb./yy2q.*(yy2q.*cosb+y3+aa)+oh.*a.*(y1.*cosb+y3aa.*sinb)./yy2.^(th).*(yy2q.*cosb+y3+aa).*(2.*y3+4.*a)-a.*(y1.*cosb+y3aa.*sinb)./yy2q.*(oh./yy2q.*cosb.*(2.*y3+4.*a)+1))))./pi./(1-nu))+oh.*B1.*(oe.*(1./sysq-1./yy2q-cosb.*((sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb)-(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)))./pi./(1-nu)+oe.*y2.*(-1./(sys).^(th).*y2+1./yy2.^(th).*y2-cosb.*(1./(sys).*cosb.*y2./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(sysq.*cosb-y3)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-1./yy2.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2+(yy2q.*cosb+y3+aa)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2))./pi./(1-nu)+1./4.*((2-nunu).*((1-nunu).*(-1./y1./(1+y22./y12)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2)).*cotb+1./(yy2q+y3+aa).*(nunu+a./yy2q)-y22./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q-y22./(yy2q+y3+aa).*a./yy2.^(th)-cosb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+y22.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q+y22.*cosb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th))+(y3+a)./yy2q.*(nunu./(yy2q+y3+aa)+a./yy2)-y22.*(y3+a)./yy2.^(th).*(nunu./(yy2q+y3+aa)+a./yy2)+y2.*(y3+a)./yy2q.*(-nunu./(yy2q+y3+aa).^2./yy2q.*y2-aa./yy2.^2.*y2)+(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2)-y22.*(y3+a).*cosb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2)-y22.*(y3+a).*cosb./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1-nunu-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-a.*y3aa./yy2)+y2.*(y3+a).*cosb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(-1./yy2q.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2+aa.*y3aa./yy2.^2.*y2))./pi./(1-nu))+oh.*B2.*(oe.*((-1+nunu).*sinb.*(1./sysq.*y2./(sysq-y1.*sinb-y3.*cosb)-1./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb))-y1.*(-1./(sys).^(th).*y2+1./yy2.^(th).*y2)+(y1.*cosb-y3.*sinb)./(sys).*cosb.*y2./(sysq-y1.*sinb-y3.*cosb)-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(y1.*cosb-y3.*sinb).*(sysq.*cosb-y3)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2)./pi./(1-nu)+1./4.*((-2+nunu).*(1-nunu).*cotb.*(1./yy2q.*y2./(yy2q+y3+aa)-cosb./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb))+(2-nunu).*y1./(yy2q+y3+aa).^2.*(nunu+a./yy2q)./yy2q.*y2+(2-nunu).*y1./(yy2q+y3+aa).*a./yy2.^(th).*y2-(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2-(2-nunu).*(y1.*cosb+y3aa.*sinb)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2-(y3+a)./yy2.^(th).*((1-nunu).*cotb-nunu.*y1./(yy2q+y3+aa)-a.*y1./yy2).*y2+(y3+a)./yy2q.*(nunu.*y1./(yy2q+y3+aa).^2./yy2q.*y2+aa.*y1./yy2.^2.*y2)+(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb.*sinb+(yy2q.*cosb+y3+aa).*cotb./yy2q.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))+a./yy2q.*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)))./yy2q.*y2-(y3+a)./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2.*cosb.*y2.*cotb.*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb))-(yy2q.*cosb+y3+aa).*cotb./yy2.^(th).*((2-nunu).*cosb-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb)).*y2+(yy2q.*cosb+y3+aa).*cotb./yy2q.*(-cosb./yy2q.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2./yy2q.*y2)-a./yy2.^(th).*(sinb-y3aa.*(y1.*cosb+y3aa.*sinb)./yy2-(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb)).*y2+a./yy2q.*(2.*y3aa.*(y1.*cosb+y3aa.*sinb)./yy2.^2.*y2-(y1.*cosb+y3aa.*sinb)./yy2.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2+(y1.*cosb+y3aa.*sinb).*(yy2q.*cosb+y3+aa)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2)))./pi./(1-nu))+oh.*B3.*(oe.*((2-nunu).*(1./(y1.*cosb-y3.*sinb)./(1+y22./(y1.*cosb-y3.*sinb).^2)+(sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)+y22./sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb)-2.*y22.*sysq.*sinb./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*(sys).*sinb.^2./(y1.*(y1.*cosb-y3.*sinb)+y22.*cosb).^2)-1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)-(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+sinb.*((sysq.*cosb-y3)./sysq./(sysq-y1.*sinb-y3.*cosb)-(yy2q.*cosb+y3+aa)./yy2q./(yy2q-y1.*sinb+y3aa.*cosb))+y2.*sinb.*(1./(sys).*cosb.*y2./(sysq-y1.*sinb-y3.*cosb)-(sysq.*cosb-y3)./(sys).^(th)./(sysq-y1.*sinb-y3.*cosb).*y2-(sysq.*cosb-y3)./(sys)./(sysq-y1.*sinb-y3.*cosb).^2.*y2-1./yy2.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb)+(yy2q.*cosb+y3+aa)./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*y2+(yy2q.*cosb+y3+aa)./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*y2))./pi./(1-nu)+1./4.*((2-nunu).*(-1./y1./(1+y22./y12)+1./(y1.*cosb+y3aa.*sinb)./(1+y22./(y1.*cosb+y3aa.*sinb).^2)+(yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)+y22./yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb)-2.*y22.*yy2q.*sinb./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2.*cosb)./(1+y22.*yy2.*sinb.^2./(y1.*(y1.*cosb+y3aa.*sinb)+y22.*cosb).^2))+(2-nunu).*sinb./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(2-nunu).*y22.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q-(2-nunu).*y22.*sinb./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th)+(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-y22.*(y3+a).*sinb./yy2.^(th)./(yy2q-y1.*sinb+y3aa.*cosb).*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)-y22.*(y3+a).*sinb./yy2./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(1+(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)+a.*y3aa./yy2)+y2.*(y3+a).*sinb./yy2q./(yy2q-y1.*sinb+y3aa.*cosb).*(1./yy2q.*cosb.*y2./(yy2q-y1.*sinb+y3aa.*cosb).*(cosb+a./yy2q)-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).^2.*(cosb+a./yy2q)./yy2q.*y2-(yy2q.*cosb+y3+aa)./(yy2q-y1.*sinb+y3aa.*cosb).*a./yy2.^(th).*y2-aa.*y3aa./yy2.^2.*y2))./pi./(1-nu));
